﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ServiceModel;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.ResponseObjects;

namespace HeuristicLab.Hive.Contracts.Interfaces {
  [ServiceContract]
  public interface IClientFacade {
    [OperationContract]
    ResponseObject<JobDto> AddJob(SerializedJob job);
    [OperationContract]
    ResponseObject<JobDto> AddJobWithGroupStrings(SerializedJob jobObj, IEnumerable<string> groups);
    [OperationContract]
    Response RequestSnapshot(Guid jobId);
    [OperationContract]
    ResponseObject<SerializedJob> GetLastSerializedResult(Guid jobId);
    [OperationContract]
    ResponseObject<SerializedJob> GetSnapshotResult(Guid jobId);
    [OperationContract]
    Response AbortJob(Guid jobId);
    [OperationContract]
    ResponseObject<JobDto> GetJobById(Guid jobId);
    [OperationContract]
    ResponseObject<JobResultList> GetJobResults(IEnumerable<Guid> jobIds);
    [OperationContract]
    ResponseObject<JobResultList> GetChildJobResults(Guid? parentJobId, bool recursive, bool includeParent);

    /// <summary>
    /// Returns all experiments for the current user
    /// </summary>
    [OperationContract]
    ResponseObject<HiveExperimentDtoList> GetHiveExperiments();
    
    [OperationContract]
    ResponseObject<HiveExperimentDto> UpdateHiveExperiment(HiveExperimentDto hiveExperimentDto);

    [OperationContract]
    Response DeleteHiveExperiment(Guid hiveExperimentId);

    [OperationContract]
    ResponseObject<JobDto> AddChildJob(Guid parentJobId, SerializedJob serializedJob);
  }
}
