﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Transactions;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;

namespace HeuristicLab.Hive.Server.Core {
  internal class DefaultScheduler : IScheduler {

    private static object jobLock = new object();

    #region IScheduler Members

    public DefaultScheduler() { }

    public bool ExistsJobForSlave(HeartBeatData hbData) {
      return 
        DaoLocator.JobDao.FindJobsWithFinishedChilds(hbData.SlaveId).Count() > 0 || 
        DaoLocator.JobDao.FindFittingJobs(JobState.Offline, hbData.FreeCores, hbData.FreeMemory, hbData.SlaveId).Count() > 0;
    }

    public JobDto GetNextJobForSlave(Guid slaveId) {
      lock (jobLock) {
        /// Critical section ///
        JobDto jobToCalculate = null;
        using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = ApplicationConstants.ISOLATION_LEVEL_SCOPE })) {
          SlaveDto slave = DaoLocator.SlaveDao.FindById(slaveId);

          if (slave != null) {
            LinkedList<JobDto> finishedJobsForSlave = new LinkedList<JobDto>(DaoLocator.JobDao.FindJobsWithFinishedChilds(slave.Id));
            if (finishedJobsForSlave != null && finishedJobsForSlave.Count > 0) {
              jobToCalculate = finishedJobsForSlave.First.Value;
              UpdateJobAsCalculating(jobToCalculate, slave);
            }

            if (slave.NrOfFreeCores.HasValue && slave.FreeMemory.HasValue) {
              LinkedList<JobDto> offlineJobsForSlave = new LinkedList<JobDto>(DaoLocator.JobDao.FindFittingJobs(JobState.Offline, slave.NrOfFreeCores.Value, slave.FreeMemory.Value, slave.Id));
              if (offlineJobsForSlave != null && offlineJobsForSlave.Count > 0) {
                jobToCalculate = offlineJobsForSlave.First.Value;
                UpdateJobAsCalculating(jobToCalculate, slave);
              }
            }
          }
          scope.Complete();
        }
        /// End Critical section ///

        return jobToCalculate;
      }
    }

    private static void UpdateJobAsCalculating(JobDto jobToCalculate, SlaveDto slave) {
      jobToCalculate.State = JobState.Calculating;
      jobToCalculate.Slave = slave;
      jobToCalculate.Slave.State = SlaveState.Calculating;
      jobToCalculate.DateCalculated = DateTime.Now;
      DaoLocator.JobDao.AssignSlaveToJob(slave.Id, jobToCalculate.Id);
      DaoLocator.JobDao.Update(jobToCalculate);
      DaoLocator.SlaveDao.Update(jobToCalculate.Slave);
    }

    #endregion
  }
}